package controllers;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JFrame;

import models.ObservableDomino;
import specifications.DominoSpecification;


/**
 * A controller for a domino ({@link DominoSpecification} providing 2 buttons in its own frame:
 * <ul>
 * <li> switching the domino left and right values </li>
 * <li> setting the left and right values randomly </li>
 * </ul>
 * It is a controller in the MVC design {@link applications.DominoMVC}
 * @author J Paul Gibson
 * @version 1
 *
 */
public class DominoButtonController implements ActionListener{
	
	/**
	 * The width of the frame should be big enough for the button text horizontally
	 */
	  final static int WIDTH = 200;
	
	/**
	 * The height of the frame should be big enough for two buttons arranged vertically
	 */
	  final static int HEIGHT = 100;
	  
	  /**
	   * The label in the randomize button
	   */
	  final static String RANDOMIZE = "Randomize";
	  
	  /**
	   * The label in the switch button
	   */
	  final static String SWITCH = "Switch";
	
	    /**
	     * The frame in which the domino button will be displayed
	     */
	    JFrame frame;

	    /**
	     * The button for randomizing the domino
	     */
		JButton buttonRandomizeDomino;
		
		/**
	     * The button for switching the domino
	     */
		JButton buttonSwitchDomino;
		
		/**
		 * The domino to be randomized when the button is pressed
		 */
		ObservableDomino domino;
		
		/**
		 * A 2-button controller for a domino:
		 * <ul>
		 * <li> Button for switching the domino around (swapping left and right values) </li>
		 * <li> Button for randomly selecting a new domino </li>
		 * </ul>
		 * @param d is the domino model to which the button controller is connected 
		 * @param i horizontal screen position
		 * @param j vertical screen position
		 */
		public DominoButtonController(ObservableDomino d, int i, int j){
			 
			super();
			domino = d;
			 
			 // Initialise the frame graphics properties
			 frame = new JFrame("DominoController");
			 frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
			 frame.setSize(200,80);
			 frame.setLocation(i,j);	 
			 frame.setResizable(false);
			 
			 // Initialise the button graphics
			 buttonRandomizeDomino = new JButton(RANDOMIZE);
			 
			// Initialise the button graphics
			 buttonSwitchDomino = new JButton(SWITCH);
			 
			 // The button listens and reacts to its own event presses
			 buttonRandomizeDomino.addActionListener(this);
			 
			 // The button listens and reacts to its own event presses
			 buttonSwitchDomino.addActionListener(this);

			 // Add the button to the center of the frame's graphics pane
			 Container contentPane = frame.getContentPane();
			 contentPane.add(buttonRandomizeDomino, BorderLayout.NORTH);
			 contentPane.add(buttonSwitchDomino, BorderLayout.SOUTH);
			 frame.setVisible(true);
		}

		/**
		 * Actions performed when either button is pressed
		 * <ul>
		 * <li> RANDOMIZE - replace current domino with a new randomly generate domino </li>
		 * <li> SWITCH - flip the values on the left and right sides of the current domino </li>
		 * </ul>
		 */
	      public void actionPerformed(ActionEvent actionEvent) {
		    if (actionEvent.getActionCommand().equals(RANDOMIZE))
		    		domino.randomize();
		    else if (actionEvent.getActionCommand().equals(SWITCH))
	    		domino.switchSides();
		    
		
 } 
	      	      
}

