package tests;

import java.util.Random;

import org.junit.Assert;
import org.junit.Test;

import specifications.DominoSpecification;
import models.Domino;

/**
 * Unit tests for {@link Domino} as specified by {@link tests.JUnit_DominoSpecification}.
 * @author jpaulgibson
 * @version 1
 */
public class JUnit_Domino extends JUnit_DominoSpecification{

	@Override
	public void setUp() {
		
		dominoUnderTest = new Domino(DominoSpecification.MIN, DominoSpecification.MAX);
		doubleMax = new Domino(DominoSpecification.MAX, DominoSpecification.MAX);	
	}
	
	/**
	 * Unit test for the default constructor {@link Domino#Domino()}
	 */
	@Test
	public void test_DefaultConstructor(){
		
		dominoUnderTest = new Domino();
		Assert.assertEquals(Domino.DEFAULT_LEFT_VALUE, dominoUnderTest.getLeftValue());
		Assert.assertEquals(Domino.DEFAULT_RIGHT_VALUE, dominoUnderTest.getRightValue());
		Assert.assertTrue(dominoUnderTest.invariant());
	}
	
	/**
	 * Unit test for the copy constructor {@link Domino#Domino(DominoSpecification)} non exception case
	 */
	@Test
	public void test_CopyConstructorOK(){
		
		Assert.assertNotEquals(doubleMax, dominoUnderTest);
		dominoUnderTest = new Domino(doubleMax);
		Assert.assertEquals(doubleMax, dominoUnderTest);
		Assert.assertEquals(doubleMax.getLeftValue(), dominoUnderTest.getLeftValue());
		Assert.assertEquals(doubleMax.getRightValue(), dominoUnderTest.getRightValue());
		Assert.assertTrue(dominoUnderTest.invariant());
	}
	
	/**
	 * Unit test for the copy constructor {@link Domino#Domino(DominoSpecification)} exception case
	 */
	@Test(expected = IllegalArgumentException.class)
	public void test_CopyConstructorKO(){
		
		Domino domino=null;
		dominoUnderTest = new Domino(domino);

	}
	
	/**
	 * Unit test for the constructor {@link Domino#Domino(int, int)}
	 */
	@Test
	public void test_Constructor(){
		
		for (int leftValue = DominoSpecification.MIN; leftValue<=DominoSpecification.MAX; leftValue++)
			for (int rightValue = DominoSpecification.MIN; rightValue<=DominoSpecification.MAX; rightValue++){
				dominoUnderTest = new Domino(leftValue, rightValue);
				Assert.assertTrue(dominoUnderTest.invariant());
				
			}			
	}
	
	/**
	 * Unit test for the constructor {@link Domino#Domino(Random)}: make a random domino 1000 times and check that
	 * it is always valid.<br>
	 * The 'randomness' of the constructor is tested by the validation test {@link ValidationDomino}.
	 */
	@Test
	public void test_RandomConstructor(){
		
		final int NUMBER_OF_RANDOM_TESTS = 1000;
		
		Random rng = new Random();
		
		
		for (int testCount = 0; testCount < NUMBER_OF_RANDOM_TESTS; testCount++){
				dominoUnderTest = new Domino(rng);
				Assert.assertTrue(dominoUnderTest.invariant());	
		}			
	}
	
	/**
	 * Unit test for the constructor {@link Domino#Domino(int, int)} exception case
	 */
	@Test(expected = IllegalArgumentException.class)
	public void test_ConstructorException(){
		
			dominoUnderTest = new Domino(DominoSpecification.MIN-1, DominoSpecification.MIN);
			dominoUnderTest = new Domino(DominoSpecification.MIN, DominoSpecification.MIN-1);
			dominoUnderTest = new Domino(DominoSpecification.MAX+1, DominoSpecification.MAX);
			dominoUnderTest = new Domino(DominoSpecification.MAX, DominoSpecification.MAX+1);
	}

}
