package tests;

import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import specifications.DominoSpecification;

/**
 * Unit tests for {@link DominoSpecification}
 * @author jpaulgibson
 * @version 1
 */
public abstract class JUnit_DominoSpecification {
	
	DominoSpecification dominoUnderTest;
    DominoSpecification doubleMax;
	
	/**
	 * Should initialise dominoUnderTest to "MIN: MAX" and doubleMax to "MAX : MAX"
	 */
	@Before
	public abstract void setUp();
	
	@After
	public  void tearDown(){
		dominoUnderTest =null;
	}
	
	/**
	 *  Check that the concrete setUp method in the concrete test class initialises the test objects correctly,
	 *  as defined in {@link JUnit_DominoSpecification#setUp}.<br>
	 *  It also checks that the constructors respect the invariant.
	 */
	@Test
	public void test_setUp(){
		Assert.assertEquals(DominoSpecification.MIN, dominoUnderTest.getLeftValue());
		Assert.assertEquals(DominoSpecification.MAX, dominoUnderTest.getRightValue());
		Assert.assertEquals(DominoSpecification.MAX, doubleMax.getLeftValue());
		Assert.assertEquals(DominoSpecification.MAX, doubleMax.getRightValue());
		Assert.assertTrue(dominoUnderTest.invariant());
		Assert.assertTrue(doubleMax.invariant());
	}
	
	/**
	 *  Unit test for {@link DominoSpecification#getLeftValue}
	 */
	@Test
	public void test_getLeft(){
		
		Assert.assertEquals(DominoSpecification.MIN, dominoUnderTest.getLeftValue());
		Assert.assertEquals(DominoSpecification.MAX, doubleMax.getLeftValue());	
	}
	
	/**
	 *  Unit test for {@link DominoSpecification#getRightValue}
	 */
	@Test
	public void test_getRight(){
		
		Assert.assertEquals(DominoSpecification.MAX, dominoUnderTest.getRightValue());
		Assert.assertEquals(DominoSpecification.MAX, doubleMax.getRightValue());	
	}
	
	/**
	 *  Unit test for {@link DominoSpecification#switchSides}.
	 */
	@Test
	public  void test_switchSides(){
		
		int left = dominoUnderTest.getLeftValue();
		int right = dominoUnderTest.getRightValue();
		
		dominoUnderTest.switchSides();
		Assert.assertTrue(dominoUnderTest.invariant());
		
		Assert.assertEquals(right, dominoUnderTest.getLeftValue());
		Assert.assertEquals(left, dominoUnderTest.getRightValue());
	}
	
	/**
	 *  Unit test for {@link DominoSpecification#equals}.
	 */
	@Test
	public  void test_equals(){
		
		Assert.assertNotEquals(dominoUnderTest, null);
		Assert.assertNotEquals(dominoUnderTest, "Not a domino");
		
		Assert.assertEquals(dominoUnderTest, dominoUnderTest);
		
		DominoSpecification copy = dominoUnderTest.copy();
		Assert.assertEquals(dominoUnderTest, copy);
		
		copy.switchSides();
		Assert.assertEquals(dominoUnderTest, copy);
		
		Assert.assertNotEquals(dominoUnderTest, doubleMax);
		
	}
	
	/**
	 * Unit test for {@link DominoSpecification#hashCode}
	 */
	@Test 
	public void test_hashCode(){
		
	    int hash = dominoUnderTest.hashCode();
		Assert.assertNotEquals(hash, doubleMax.hashCode());
		dominoUnderTest.switchSides();
		Assert.assertEquals(hash, dominoUnderTest.hashCode());
		
	}
	
	
	/**
	 * Unit test for {@link DominoSpecification#toString}.
	 */
	@Test
	public void test_toString(){
		Assert.assertEquals(DominoSpecification.MIN+" : "+DominoSpecification.MAX,dominoUnderTest.toString());
		Assert.assertEquals(DominoSpecification.MAX+" : "+DominoSpecification.MAX,doubleMax.toString());
		
		dominoUnderTest.switchSides();
		Assert.assertEquals(DominoSpecification.MAX+" : "+DominoSpecification.MIN,dominoUnderTest.toString());
	}
	
	/**
	 * Unit test for {@link DominoSpecification#copy}.
	 */
	@Test
	public void test_copy(){
		
		DominoSpecification copy = dominoUnderTest.copy();
		Assert.assertEquals(dominoUnderTest, copy);
	    Assert.assertEquals(copy, dominoUnderTest);
	   // Assert.assertEquals(copy.hashCode(), dominoUnderTest.hashCode());
		Assert.assertTrue(copy.invariant());

	}
	
	
}
