package tools;

import java.util.Random;

/**
 * Interface for classes that have a randomize method implemented
 * @author jpaulgibson
 * @version 1
 */
public interface Randomizable {

	/**
	 * update the state of the object in a random (safe) manner. Use a local, temporary <b>RNG</b>
	 * @throws InvariantBrokenException if the domino state is not valid after execution (this should not happen)
	 */
	public void randomize() throws InvariantBrokenException ;
	
	/**
	 * update the state of the object in a random (safe) manner
	 * @param rng the random number generator used to randomly set the domino values, if this is null then we construct it<br>
	 * @throws InvariantBrokenException if the domino state is not valid after execution (this should not happen)
	 */
	public void randomize(Random rng) throws InvariantBrokenException ;
}


