package views;

import java.awt.BorderLayout;
import java.awt.Container;
import javax.swing.JFrame;
import models.ObservableDomino;

/**
 * The frame in which a domino image will be drawn in a single canvas (the same size as the frame)
 * @author jpaulgibson
 * @version 1
 */
public class DominoFrame extends JFrame {

	private static final long serialVersionUID = 1L;
	
	/**
	 * The single panel in the frame used to draw a single domino
	 */
	private DominoPanel  dominoPanel;
	
	/**
	 * The width of the frame should be bigger than the width of the domino image being displayed
	 * ({@link views.DominoImages#IMAGE_WIDTH})
	 */
	public static final int WIDTH = 200;
	
	/**
	 * The height of the frame should be bigger than the height of the domino image being displayed
	 * ({@link views.DominoImages#IMAGE_HEIGHT})
	 */
	public static final int HEIGHT = 100;

	/**
	 * @param domino is the domino model to which the view listens<br>
	 * @param hor initial horizontal screen position of top left of frame
	 * @param ver initial vertical screen position of top left of frame
	 */
	public DominoFrame(ObservableDomino domino, int hor, int ver) {
			
		    super(); 
		    setTitle("Domino");
		    dominoPanel = new DominoPanel (domino, WIDTH, HEIGHT );
			Container contentPane = getContentPane();
			contentPane.add(dominoPanel, BorderLayout.CENTER );
		    domino.addObserver(dominoPanel);
		    
		    setSize(WIDTH,HEIGHT);
		    setLocation(hor,ver);
		    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		    setResizable(false);
		    
			setVisible(true);
				 
		}
		


	}

